//
// Created by spenc on 5/13/2025.
//

#pragma once

#include <cmath>
#include <cstring>  // memcpy
#include "Vec3.h"

namespace OX {

struct alignas(16) Mat4 {
    float m[16];

    // Constructs identity
    Mat4() {
        LoadIdentity();
    }

    explicit Mat4(float diagonal) {
        LoadIdentity();
        m[0] = m[5] = m[10] = m[15] = diagonal;
    }

    void LoadIdentity() {
        std::memset(m, 0, sizeof(m));
        m[0] = m[5] = m[10] = m[15] = 1.0f;
    }

    float* Ptr() { return m; }
    const float* Ptr() const { return m; }

    // Matrix multiplication (Mat4 * Mat4)
    Mat4 operator*(const Mat4& rhs) const {
        Mat4 result;
        for (int col = 0; col < 4; ++col) {
            for (int row = 0; row < 4; ++row) {
                result.m[col * 4 + row] =
                    m[0 * 4 + row] * rhs.m[col * 4 + 0] +
                    m[1 * 4 + row] * rhs.m[col * 4 + 1] +
                    m[2 * 4 + row] * rhs.m[col * 4 + 2] +
                    m[3 * 4 + row] * rhs.m[col * 4 + 3];
            }
        }
        return result;
    }

    // Create translation matrix
    static Mat4 Translate(const Vec3& t) {
        Mat4 result;
        result.m[12] = t.x;
        result.m[13] = t.y;
        result.m[14] = t.z;
        return result;
    }

    // Create scale matrix
    static Mat4 Scale(const Vec3& s) {
        Mat4 result;
        result.m[0]  = s.x;
        result.m[5]  = s.y;
        result.m[10] = s.z;
        return result;
    }

    // Rotation around X axis
    static Mat4 RotateX(float angleRadians) {
        Mat4 result;
        float c = std::cos(angleRadians);
        float s = std::sin(angleRadians);
        result.m[5] = c;
        result.m[6] = s;
        result.m[9] = -s;
        result.m[10] = c;
        return result;
    }

    // Rotation around Y axis
    static Mat4 RotateY(float angleRadians) {
        Mat4 result;
        float c = std::cos(angleRadians);
        float s = std::sin(angleRadians);
        result.m[0] = c;
        result.m[2] = -s;
        result.m[8] = s;
        result.m[10] = c;
        return result;
    }

    // Rotation around Z axis
    static Mat4 RotateZ(float angleRadians) {
        Mat4 result;
        float c = std::cos(angleRadians);
        float s = std::sin(angleRadians);
        result.m[0] = c;
        result.m[1] = s;
        result.m[4] = -s;
        result.m[5] = c;
        return result;
    }

    static Mat4 Identity() {
        return Mat4();
    }


    Mat4 Inverse() const {
        float inv[16];

        // Cofactor matrix
        #define MAT(r, c) m[(c) * 4 + (r)]
        #define INV(r, c) inv[(c) * 4 + (r)]

        INV(0,0) =  MAT(1,1) * MAT(2,2) * MAT(3,3) - MAT(1,1) * MAT(2,3) * MAT(3,2) -
                    MAT(2,1) * MAT(1,2) * MAT(3,3) + MAT(2,1) * MAT(1,3) * MAT(3,2) +
                    MAT(3,1) * MAT(1,2) * MAT(2,3) - MAT(3,1) * MAT(1,3) * MAT(2,2);

        INV(0,1) = -MAT(0,1) * MAT(2,2) * MAT(3,3) + MAT(0,1) * MAT(2,3) * MAT(3,2) +
                    MAT(2,1) * MAT(0,2) * MAT(3,3) - MAT(2,1) * MAT(0,3) * MAT(3,2) -
                    MAT(3,1) * MAT(0,2) * MAT(2,3) + MAT(3,1) * MAT(0,3) * MAT(2,2);

        INV(0,2) =  MAT(0,1) * MAT(1,2) * MAT(3,3) - MAT(0,1) * MAT(1,3) * MAT(3,2) -
                    MAT(1,1) * MAT(0,2) * MAT(3,3) + MAT(1,1) * MAT(0,3) * MAT(3,2) +
                    MAT(3,1) * MAT(0,2) * MAT(1,3) - MAT(3,1) * MAT(0,3) * MAT(1,2);

        INV(0,3) = -MAT(0,1) * MAT(1,2) * MAT(2,3) + MAT(0,1) * MAT(1,3) * MAT(2,2) +
                    MAT(1,1) * MAT(0,2) * MAT(2,3) - MAT(1,1) * MAT(0,3) * MAT(2,2) -
                    MAT(2,1) * MAT(0,2) * MAT(1,3) + MAT(2,1) * MAT(0,3) * MAT(1,2);

        INV(1,0) = -MAT(1,0) * MAT(2,2) * MAT(3,3) + MAT(1,0) * MAT(2,3) * MAT(3,2) +
                    MAT(2,0) * MAT(1,2) * MAT(3,3) - MAT(2,0) * MAT(1,3) * MAT(3,2) -
                    MAT(3,0) * MAT(1,2) * MAT(2,3) + MAT(3,0) * MAT(1,3) * MAT(2,2);

        INV(1,1) =  MAT(0,0) * MAT(2,2) * MAT(3,3) - MAT(0,0) * MAT(2,3) * MAT(3,2) -
                    MAT(2,0) * MAT(0,2) * MAT(3,3) + MAT(2,0) * MAT(0,3) * MAT(3,2) +
                    MAT(3,0) * MAT(0,2) * MAT(2,3) - MAT(3,0) * MAT(0,3) * MAT(2,2);

        INV(1,2) = -MAT(0,0) * MAT(1,2) * MAT(3,3) + MAT(0,0) * MAT(1,3) * MAT(3,2) +
                    MAT(1,0) * MAT(0,2) * MAT(3,3) - MAT(1,0) * MAT(0,3) * MAT(3,2) -
                    MAT(3,0) * MAT(0,2) * MAT(1,3) + MAT(3,0) * MAT(0,3) * MAT(1,2);

        INV(1,3) =  MAT(0,0) * MAT(1,2) * MAT(2,3) - MAT(0,0) * MAT(1,3) * MAT(2,2) -
                    MAT(1,0) * MAT(0,2) * MAT(2,3) + MAT(1,0) * MAT(0,3) * MAT(2,2) +
                    MAT(2,0) * MAT(0,2) * MAT(1,3) - MAT(2,0) * MAT(0,3) * MAT(1,2);

        INV(2,0) =  MAT(1,0) * MAT(2,1) * MAT(3,3) - MAT(1,0) * MAT(2,3) * MAT(3,1) -
                    MAT(2,0) * MAT(1,1) * MAT(3,3) + MAT(2,0) * MAT(1,3) * MAT(3,1) +
                    MAT(3,0) * MAT(1,1) * MAT(2,3) - MAT(3,0) * MAT(1,3) * MAT(2,1);

        INV(2,1) = -MAT(0,0) * MAT(2,1) * MAT(3,3) + MAT(0,0) * MAT(2,3) * MAT(3,1) +
                    MAT(2,0) * MAT(0,1) * MAT(3,3) - MAT(2,0) * MAT(0,3) * MAT(3,1) -
                    MAT(3,0) * MAT(0,1) * MAT(2,3) + MAT(3,0) * MAT(0,3) * MAT(2,1);

        INV(2,2) =  MAT(0,0) * MAT(1,1) * MAT(3,3) - MAT(0,0) * MAT(1,3) * MAT(3,1) -
                    MAT(1,0) * MAT(0,1) * MAT(3,3) + MAT(1,0) * MAT(0,3) * MAT(3,1) +
                    MAT(3,0) * MAT(0,1) * MAT(1,3) - MAT(3,0) * MAT(0,3) * MAT(1,1);

        INV(2,3) = -MAT(0,0) * MAT(1,1) * MAT(2,3) + MAT(0,0) * MAT(1,3) * MAT(2,1) +
                    MAT(1,0) * MAT(0,1) * MAT(2,3) - MAT(1,0) * MAT(0,3) * MAT(2,1) -
                    MAT(2,0) * MAT(0,1) * MAT(1,3) + MAT(2,0) * MAT(0,3) * MAT(1,1);

        INV(3,0) = -MAT(1,0) * MAT(2,1) * MAT(3,2) + MAT(1,0) * MAT(2,2) * MAT(3,1) +
                    MAT(2,0) * MAT(1,1) * MAT(3,2) - MAT(2,0) * MAT(1,2) * MAT(3,1) -
                    MAT(3,0) * MAT(1,1) * MAT(2,2) + MAT(3,0) * MAT(1,2) * MAT(2,1);

        INV(3,1) =  MAT(0,0) * MAT(2,1) * MAT(3,2) - MAT(0,0) * MAT(2,2) * MAT(3,1) -
                    MAT(2,0) * MAT(0,1) * MAT(3,2) + MAT(2,0) * MAT(0,2) * MAT(3,1) +
                    MAT(3,0) * MAT(0,1) * MAT(2,2) - MAT(3,0) * MAT(0,2) * MAT(2,1);

        INV(3,2) = -MAT(0,0) * MAT(1,1) * MAT(3,2) + MAT(0,0) * MAT(1,2) * MAT(3,1) +
                    MAT(1,0) * MAT(0,1) * MAT(3,2) - MAT(1,0) * MAT(0,2) * MAT(3,1) -
                    MAT(3,0) * MAT(0,1) * MAT(1,2) + MAT(3,0) * MAT(0,2) * MAT(1,1);

        INV(3,3) =  MAT(0,0) * MAT(1,1) * MAT(2,2) - MAT(0,0) * MAT(1,2) * MAT(2,1) -
                    MAT(1,0) * MAT(0,1) * MAT(2,2) + MAT(1,0) * MAT(0,2) * MAT(2,1) +
                    MAT(2,0) * MAT(0,1) * MAT(1,2) - MAT(2,0) * MAT(0,2) * MAT(1,1);

        #undef MAT
        #undef INV

        float det = m[0] * inv[0] + m[1] * inv[4] + m[2] * inv[8] + m[3] * inv[12];
        if (det == 0.0f) return Mat4::Identity();

        float invDet = 1.0f / det;
        Mat4 result;
        for (int i = 0; i < 16; ++i)
            result.m[i] = inv[i] * invDet;

        return result;
    }


};

};

